Package: clusterCrit
Version: 1.3.0
License: GPL (>= 2)
Description: Package providing functions for computing a collection of clustering validation or quality criteria and partition comparison indices.
Title: Clustering Indices
Authors@R: c(
  person("Iago", "Giné-Vázquez", role = c("cre"), 
                                 email = "iago.gin-vaz@protonmail.com", 
                                 comment = c(ORCID = "0000-0002-6725-2638")),
  person("Bernard", "Desgraupes", role = "aut"))
Date: 2023-11-21
Suggests: RUnit, rbenchmark
URL: https://gitlab.com/iagogv/clusterCrit
BugReports: https://gitlab.com/iagogv/clusterCrit/-/issues
Collate: main.R zzz.R
NeedsCompilation: yes
Encoding: UTF-8
Type: Package
Packaged: 2023-11-21 23:09:51 UTC; iago
Author: Iago Giné-Vázquez [cre] (<https://orcid.org/0000-0002-6725-2638>),
  Bernard Desgraupes [aut]
Maintainer: Iago Giné-Vázquez <iago.gin-vaz@protonmail.com>
Repository: CRAN
Date/Publication: 2023-11-23 16:00:05 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 14:49:01 UTC; unix
Archs: clusterCrit.so.dSYM
