**clusterCrit**


[![CRAN status](https://badges.cranchecks.info/summary/clusterCrit.svg)](https://cran.r-project.org/web/checks/check_results_clusterCrit.html)

In words of the author of the package, Bernard Desgraupes, whose personal webpage [https://bdesgraupes.pagesperso-orange.fr/](https://web.archive.org/web/20220305125743/https://bdesgraupes.pagesperso-orange.fr/)  is accessible through [Internet Archive](https://web.archive.org/).

> This package computes clusterization quality indices. It supports 42 internal indices (Calinski_Harabasz, Dunn, Ray_Turi, etc.) for cluster validation and 14 external indices (Phi, Jaccard, McNemar, etc.) to compare partitions.
>
> A vignette is included and gives the exact definition of all the indices.
> 
> The package is written internally in Fortran and in C so you will need a Fortran and a C compiler if you want to compile it from sources. Alternatively, there are already compiled binary versions on the CRAN mirrors.

